@@ -18,10 +18,14 @@ ModuleHeader MOD_HEADER
 struct geoip_classic_config_s {
 	char *v4_db_file;
 	char *v6_db_file;
+	char *asn_v4_db_file;
+	char *asn_v6_db_file;
 /* for config reading only */
 	int have_config;
 	int have_ipv4_database;
 	int have_ipv6_database;
+	int have_asn_ipv4_database;
+	int have_asn_ipv6_database;
 };
 
 /* Variables */
@@ -29,6 +33,8 @@ struct geoip_classic_config_s {
 struct geoip_classic_config_s geoip_classic_config;
 GeoIP *gi4 = NULL;
 GeoIP *gi6 = NULL;
+GeoIP *asn_gi4 = NULL;
+GeoIP *asn_gi6 = NULL;
 
 /* Forward declarations */
 int geoip_classic_configtest(ConfigFile *cf, ConfigEntry *ce, int type, int *errs);
@@ -88,6 +94,38 @@ int geoip_classic_configtest(ConfigFile *cf, ConfigEntry *ce, int type, int *err
 			geoip_classic_config.have_ipv6_database = 1;
 			continue;
 		}
+		if (!strcmp(cep->name, "asn-ipv4-database"))
+		{
+			if (geoip_classic_config.have_asn_ipv4_database)
+			{
+				config_error("%s:%i: duplicate item set::geoip-classic::%s", cep->file->filename, cep->line_number, cep->name);
+				continue;
+			}
+			if (!is_file_readable(cep->value, PERMDATADIR))
+			{
+				config_error("%s:%i: set::geoip-classic::%s: cannot open file \"%s/%s\" for reading (%s)", cep->file->filename, cep->line_number, cep->name, PERMDATADIR, cep->value, strerror(errno));
+				errors++;
+				continue;
+			}
+			geoip_classic_config.have_asn_ipv4_database = 1;
+			continue;
+		}
+		if (!strcmp(cep->name, "asn-ipv6-database"))
+		{
+			if (geoip_classic_config.have_asn_ipv6_database)
+			{
+				config_error("%s:%i: duplicate item set::geoip-classic::%s", cep->file->filename, cep->line_number, cep->name);
+				continue;
+			}
+			if (!is_file_readable(cep->value, PERMDATADIR))
+			{
+				config_error("%s:%i: set::geoip-classic::%s: cannot open file \"%s/%s\" for reading (%s)", cep->file->filename, cep->line_number, cep->name, PERMDATADIR, cep->value, strerror(errno));
+				errors++;
+				continue;
+			}
+			geoip_classic_config.have_asn_ipv6_database = 1;
+			continue;
+		}
 		config_warn("%s:%i: unknown item set::geoip-classic::%s", cep->file->filename, cep->line_number, cep->name);
 	}
 	
@@ -109,6 +147,8 @@ int geoip_classic_configposttest(int *errs)
 	{
 		safe_strdup(geoip_classic_config.v4_db_file, "GeoIP.dat");
 		safe_strdup(geoip_classic_config.v6_db_file, "GeoIPv6.dat");
+		safe_strdup(geoip_classic_config.asn_v4_db_file, "GeoIPASNum.dat");
+		safe_strdup(geoip_classic_config.asn_v6_db_file, "GeoIPASNumv6.dat");
 
 		if (is_file_readable(geoip_classic_config.v4_db_file, PERMDATADIR))
 		{
@@ -118,6 +158,7 @@ int geoip_classic_configposttest(int *errs)
 			config_warn("[geoip_classic] cannot open IPv4 database file \"%s/%s\" for reading (%s)", PERMDATADIR, geoip_classic_config.v4_db_file, strerror(errno));
 			safe_free(geoip_classic_config.v4_db_file);
 		}
+
 		if (is_file_readable(geoip_classic_config.v6_db_file, PERMDATADIR))
 		{
 			geoip_classic_config.have_ipv6_database = 1;
@@ -126,12 +167,32 @@ int geoip_classic_configposttest(int *errs)
 			config_warn("[geoip_classic] cannot open IPv6 database file \"%s/%s\" for reading (%s)", PERMDATADIR, geoip_classic_config.v6_db_file, strerror(errno));
 			safe_free(geoip_classic_config.v6_db_file);
 		}
+
+		if (is_file_readable(geoip_classic_config.asn_v4_db_file, PERMDATADIR))
+		{
+			geoip_classic_config.have_asn_ipv4_database = 1;
+		} else
+		{
+			config_warn("[geoip_classic] cannot open IPv4 ASN database file \"%s/%s\" for reading (%s)", PERMDATADIR, geoip_classic_config.asn_v4_db_file, strerror(errno));
+			safe_free(geoip_classic_config.v4_db_file);
+		}
+
+		if (is_file_readable(geoip_classic_config.asn_v6_db_file, PERMDATADIR))
+		{
+			geoip_classic_config.have_asn_ipv6_database = 1;
+		} else
+		{
+			config_warn("[geoip_classic] cannot open IPv6 ASN database file \"%s/%s\" for reading (%s)", PERMDATADIR, geoip_classic_config.asn_v6_db_file, strerror(errno));
+			safe_free(geoip_classic_config.v6_db_file);
+		}
+
 		if (!geoip_classic_config.have_ipv4_database && !geoip_classic_config.have_ipv6_database)
 		{
 			config_error("[geoip_classic] couldn't read any database! Either put these in %s location "
 					"or specify another in set::geoip-classic config block", PERMDATADIR);
 			errors++;
 		}
+		/* The ASN ones are optional */
 	}
 
 	*errs = errors;
@@ -155,8 +216,12 @@ int geoip_classic_configrun(ConfigFile *cf, ConfigEntry *ce, int type)
 	{
 		if (!strcmp(cep->name, "ipv4-database") && geoip_classic_config.have_ipv4_database)
 			safe_strdup(geoip_classic_config.v4_db_file, cep->value);
-		if (!strcmp(cep->name, "ipv6-database") && geoip_classic_config.have_ipv6_database)
+		else if (!strcmp(cep->name, "ipv6-database") && geoip_classic_config.have_ipv6_database)
 			safe_strdup(geoip_classic_config.v6_db_file, cep->value);
+		else if (!strcmp(cep->name, "asn-ipv4-database"))
+			safe_strdup(geoip_classic_config.asn_v4_db_file, cep->value);
+		else if (!strcmp(cep->name, "asn-ipv6-database"))
+			safe_strdup(geoip_classic_config.asn_v6_db_file, cep->value);
 	}
 	return 1;
 }
@@ -233,6 +298,37 @@ MOD_LOAD()
 					"could not open any database!");
 		return MOD_FAILED;
 	}
+
+	/* The rest is optional */
+	if (geoip_classic_config.asn_v4_db_file)
+	{
+		convert_to_absolute_path(&geoip_classic_config.asn_v4_db_file, PERMDATADIR);
+		asn_gi4 = GeoIP_open(geoip_classic_config.asn_v4_db_file, GEOIP_STANDARD | GEOIP_CHECK_CACHE | GEOIP_SILENCE);
+		if (!asn_gi4)
+		{
+			int save_err = errno;
+			unreal_log(ULOG_WARNING, "geoip_classic", "GEOIP_CANNOT_OPEN_DB", NULL,
+				       "[ASN IPv4] Could not open '$filename': $system_error",
+				       log_data_string("filename", geoip_classic_config.asn_v4_db_file),
+				       log_data_string("system_error", strerror(save_err)));
+		}
+	}
+
+	if (geoip_classic_config.asn_v6_db_file)
+	{
+		convert_to_absolute_path(&geoip_classic_config.asn_v6_db_file, PERMDATADIR);
+		asn_gi6 = GeoIP_open(geoip_classic_config.asn_v6_db_file, GEOIP_STANDARD | GEOIP_CHECK_CACHE | GEOIP_SILENCE);
+		if (!asn_gi6)
+		{
+			int save_err = errno;
+			unreal_log(ULOG_WARNING, "geoip_classic", "GEOIP_CANNOT_OPEN_DB", NULL,
+				       "[ASN IPv6] Could not open '$filename': $system_error",
+				       log_data_string("filename", geoip_classic_config.asn_v6_db_file),
+				       log_data_string("system_error", strerror(save_err)));
+		}
+	}
+
+
 	return MOD_SUCCESS;
 }
 
@@ -248,25 +344,34 @@ void geoip_classic_free(void)
 		GeoIP_delete(gi4);
 	if (gi6)
 		GeoIP_delete(gi6);
-	gi4 = NULL;
-	gi6 = NULL;
+	if (asn_gi4)
+		GeoIP_delete(asn_gi4);
+	if (asn_gi6)
+		GeoIP_delete(asn_gi6);
+	gi4 = gi6 = asn_gi4 = asn_gi6 = NULL;
 	safe_free(geoip_classic_config.v4_db_file);
 	safe_free(geoip_classic_config.v6_db_file);
+	safe_free(geoip_classic_config.asn_v4_db_file);
+	safe_free(geoip_classic_config.asn_v6_db_file);
 }
 
 GeoIPResult *geoip_lookup_classic(char *ip)
 {
 	static char buf[256];
 	const char *country_code, *country_name;
-	GeoIPLookup gl;
+	char *isp=NULL;
+	GeoIPLookup gl, asn_gl;
 	GeoIP *gi;
 	int geoid;
 	GeoIPResult *r;
+	char ipv6;
 
 	if (!ip)
 		return NULL;
 
-	if (strchr(ip, ':'))
+	ipv6 = strchr(ip, ':') ? 1 : 0;
+
+	if (ipv6)
 	{
 		if (!gi6)
 			return NULL;
@@ -285,13 +390,40 @@ GeoIPResult *geoip_lookup_classic(char *ip)
 
 	country_code = GeoIP_code_by_id(geoid);
 	country_name = GeoIP_country_name_by_id(gi, geoid);
-
 	if (!country_code || !country_name)
 		return NULL;
 
+	/* These are optional */
+	if (ipv6 && asn_gi6)
+		isp = GeoIP_name_by_name_v6_gl(asn_gi6, ip, &asn_gl);
+	else if (!ipv6 && asn_gi4)
+		isp = GeoIP_name_by_name_gl(asn_gi4, ip, &asn_gl);
+
+	/* Return the result */
 	r = safe_alloc(sizeof(GeoIPResult));
 	safe_strdup(r->country_code, country_code);
 	safe_strdup(r->country_name, country_name);
+
+	/* ASN requires some post-processing, it has the format:
+	 * "AS##### Name Of Autonomous System"
+	 * r->asn becomes the ##### number
+	 * r->asname becomes "Name Of Autonomous System"
+	 */
+	if (isp)
+	{
+		if ((isp[0] == 'A') && (isp[1] == 'S') && isdigit(isp[2]))
+		{
+			char *p;
+			r->asn = strtoul(isp+2, NULL, 10);
+			p = strchr(isp, ' ');
+			if (p)
+			{
+				skip_whitespace(&p);
+				if (*p)
+					safe_strdup(r->asname, p);
+			}
+		}
+		safe_free(isp);
+	}
 	return r;
 }
-
