@@ -104,6 +104,7 @@ struct cfgstruct {
 	long expire_time[MAXEXPIRES];
 	char *database;
 	char *db_secret;
+	int require_minimum_channel_members;
 };
 
 typedef struct ReputationEntry ReputationEntry;
@@ -155,9 +156,11 @@ EVENT(reputation_save_db_evt);
 int reputation_load_db(void);
 int reputation_save_db(void);
 int reputation_starttime_callback(void);
+void _ban_act_set_reputation(Client *client, BanAction *action);
 
 MOD_TEST()
 {
+	MARK_AS_OFFICIAL_MODULE(modinfo);
 	memcpy(&ModInf, modinfo, modinfo->size);
 	memset(&cfg, 0, sizeof(cfg));
 	memset(&test, 0, sizeof(cfg));
@@ -165,6 +168,7 @@ MOD_TEST()
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGTEST, 0, reputation_config_test);
 	HookAdd(modinfo->handle, HOOKTYPE_CONFIGPOSTTEST, 0, reputation_config_posttest);
 	CallbackAdd(modinfo->handle, CALLBACKTYPE_REPUTATION_STARTTIME, reputation_starttime_callback);
+	EfunctionAddVoid(modinfo->handle, EFUNC_BAN_ACT_SET_REPUTATION, _ban_act_set_reputation);
 	return MOD_SUCCESS;
 }
 
@@ -185,6 +189,7 @@ MOD_INIT()
 	mreq.unserialize = reputation_md_unserialize;
 	mreq.sync = 0; /* local! */
 	mreq.type = MODDATATYPE_CLIENT;
+	mreq.priority = -1000000;
 	reputation_md = ModDataAdd(modinfo->handle, mreq);
 	if (!reputation_md)
 		abort();
@@ -259,16 +264,22 @@ void reputation_config_setdefaults(struct cfgstruct *cfg)
 	/* <=2 points after 1 hour */
 	cfg->expire_score[0] = 2;
 #ifndef TEST
-	cfg->expire_time[0]   = 3600;
+	cfg->expire_time[0]  = 3600;
 #else
-	cfg->expire_time[0]   = 36;
+	cfg->expire_time[0]  = 36;
 #endif
 	/* <=6 points after 7 days */
 	cfg->expire_score[1] = 6;
-	cfg->expire_time[1]   = 86400*7;
-	/* ANY result that has not been seen for 30 days */
-	cfg->expire_score[2] = -1;
-	cfg->expire_time[2]   = 86400*30;
+	cfg->expire_time[1]  = 86400*7;
+	/* <=12 points after 30 days */
+	cfg->expire_score[2] = 12;
+	cfg->expire_time[2]  = 86400*30;
+	/* ANY result that has not been seen for 90 days */
+	cfg->expire_score[3] = -1;
+	cfg->expire_time[3]  = 86400*90;
+
+	/* The 'require' settings */
+	cfg->require_minimum_channel_members = 3;
 }
 
 void reputation_free_config(struct cfgstruct *cfg)
@@ -291,12 +302,20 @@ int reputation_config_test(ConfigFile *cf, ConfigEntry *ce, int type, int *errs)
 
 	for (cep = ce->items; cep; cep = cep->next)
 	{
+		if (!strcmp(cep->name, "score-bump-timer") || !strcmp(cep->name, "target"))
+		{
+			config_error("%s:%i: this feature is not implemented yet in this UnrealIRCd version",
+				cep->file->filename, cep->line_number);
+			errors++;
+		} else
 		if (!cep->value)
 		{
 			config_error("%s:%i: blank set::reputation::%s without value",
 				cep->file->filename, cep->line_number, cep->name);
 			errors++;
-			continue;
+		} else
+		if (!strcmp(cep->name, "score-bump-timer-minimum-channel-members"))
+		{
 		} else
 		if (!strcmp(cep->name, "database"))
 		{
@@ -339,6 +358,10 @@ int reputation_config_run(ConfigFile *cf, ConfigEntry *ce, int type)
 
 	for (cep = ce->items; cep; cep = cep->next)
 	{
+		if (!strcmp(cep->name, "score-bump-timer-minimum-channel-members"))
+		{
+			cfg.require_minimum_channel_members = atoi(cep->value);
+		} else
 		if (!strcmp(cep->name, "database"))
 		{
 			safe_strdup(cfg.database, cep->value);
@@ -860,6 +883,15 @@ EVENT(add_scores)
 		if (!ip)
 			continue;
 
+		/* Enforce set::reputation::score-bump-timer-minimum-channel-members.
+		 * Simply skip all the bumping and setting if this requirement is not met.
+		 */
+		if (cfg.require_minimum_channel_members > 0)
+		{
+			if (highest_channel_member_count(client) < cfg.require_minimum_channel_members)
+				continue;
+		}
+
 		e = find_reputation_entry(ip);
 		if (!e)
 		{
@@ -912,8 +944,16 @@ static inline int is_reputation_expired(ReputationEntry *e)
 	{
 		if (cfg.expire_time[i] == 0)
 			break; /* end of all entries */
-		if ((e->score <= cfg.expire_score[i]) && (TStime() - e->last_seen > cfg.expire_time[i]))
-			return 1;
+		if (cfg.expire_score[i] == -1)
+		{
+			/* For -1 it means ANY score will expire after this time.. */
+			if (TStime() - e->last_seen > cfg.expire_time[i])
+				return 1;
+		} else {
+			/* Otherwise we only expire if the score is less than <X> */
+			if ((e->score <= cfg.expire_score[i]) && (TStime() - e->last_seen > cfg.expire_time[i]))
+				return 1;
+		}
 	}
 	return 0;
 }
@@ -928,11 +968,12 @@ void reputation_changed_update_users(ReputationEntry *e)
 	list_for_each_entry(client, &client_list, client_node)
 	{
 		if (client->ip && !strcmp(e->ip, client->ip))
-		{
-			/* With some (possibly unneeded) care to only go forward */
-			if (Reputation(client) < e->score)
-				Reputation(client) = e->score;
-		}
+			Reputation(client) = e->score;
+	}
+	list_for_each_entry(client, &unknown_list, lclient_node)
+	{
+		if (client->ip && !strcmp(e->ip, client->ip))
+			Reputation(client) = e->score;
 	}
 }
 
@@ -1146,10 +1187,12 @@ CMD_FUNC(reputation_user_cmd)
 		sendnotice(client, "Current number of records (IP's): %d", count_reputation_records());
 		sendnotice(client, "-");
 		sendnotice(client, "Available commands:");
-		sendnotice(client, "/REPUTATION [nick]     Show reputation info about nick name");
-		sendnotice(client, "/REPUTATION [ip]       Show reputation info about IP address");
-		sendnotice(client, "/REPUTATION [channel]  List users in channel along with their reputation score");
-		sendnotice(client, "/REPUTATION <NN        List users with reputation score below value NN");
+		sendnotice(client, "/REPUTATION <nick>          Show reputation info about nick name");
+		sendnotice(client, "/REPUTATION <nick> <value>  Adjust the reputation score of the IP address of nick to <value>");
+		sendnotice(client, "/REPUTATION <ip>            Show reputation info about IP address");
+		sendnotice(client, "/REPUTATION <ip> <value>    Adjust the reputation score of the IP address to <value>");
+		sendnotice(client, "/REPUTATION <channel>       List users in channel along with their reputation score");
+		sendnotice(client, "/REPUTATION <NN             List users with reputation score below value NN");
 		return;
 	}
 
@@ -1206,6 +1249,30 @@ CMD_FUNC(reputation_user_cmd)
 		return;
 	}
 
+	if ((parc > 2) && !BadPtr(parv[2]))
+	{
+		/* Request to change value */
+		if (!ValidatePermissionsForPath("client:set:reputation",client,NULL,NULL,NULL))
+		{
+			sendnumeric(client, ERR_NOPRIVILEGES);
+			return;
+		}
+		int v = atoi(parv[2]);
+		if (v > REPUTATION_SCORE_CAP)
+			v = REPUTATION_SCORE_CAP;
+		if (v < 0)
+			v = 0;
+		e->score = v;
+		reputation_changed_update_users(e);
+		sendto_server(&me, 0, 0, NULL,
+			      ":%s REPUTATION %s *%d*",
+			      me.id,
+			      e->ip,
+			      e->score);
+		sendnotice(client, "Reputation of IP %s set to %hd", e->ip, e->score);
+		return;
+	}
+
 	sendnotice(client, "****************************************************");
 	sendnotice(client, "Reputation record for IP %s:", ip);
 	sendnotice(client, "    Score: %hd", e->score);
@@ -1234,6 +1301,14 @@ CMD_FUNC(reputation_user_cmd)
  * score is prefixed by * then the server will NEVER send back to the
  * uplink, it may only propagate. This is to prevent loops.
  *
+ * Since UnrealIRCd 6.0.2+ there is now also asterisk-score-asterisk:
+ * :server REPUTATION 1.2.3.4 *2*
+ * The leading asterisk means no reply will be sent back, ever, and the
+ * trailing asterisk will mean it is a "FORCED SET", which means that
+ * servers should set the reputation to that value, even if it is lower.
+ * This way reputation can be reduced and the reducation can be synced
+ * across servers, which was not possible before 6.0.2.
+ *
  * Note that some margin is used when deciding if the server should send
  * back score updates. This is defined by UPDATE_SCORE_MARGIN.
  * If this is for example set to 1 then a point difference of 1 will not
@@ -1247,6 +1322,7 @@ CMD_FUNC(reputation_server_cmd)
 	const char *ip;
 	int score;
 	int allow_reply;
+	int forced = 0;
 
 	/* :server REPUTATION <ip> <score> */
 	if ((parc < 3) || BadPtr(parv[2]))
@@ -1261,6 +1337,8 @@ CMD_FUNC(reputation_server_cmd)
 	{
 		allow_reply = 0;
 		score = atoi(parv[2]+1);
+		if (parv[2][1] && (parv[2][strlen(parv[2])-1] == '*'))
+			forced = 1;
 	} else {
 		allow_reply = 1;
 		score = atoi(parv[2]);
@@ -1268,6 +1346,8 @@ CMD_FUNC(reputation_server_cmd)
 
 	if (score > REPUTATION_SCORE_CAP)
 		score = REPUTATION_SCORE_CAP;
+	if (score < 0)
+		score = 0;
 
 	e = find_reputation_entry(ip);
 	if (allow_reply && e && (e->score > score) && (e->score - score > UPDATE_SCORE_MARGIN))
@@ -1295,6 +1375,18 @@ CMD_FUNC(reputation_server_cmd)
 			   log_data_string("ip", ip),
 			   log_data_integer("their_score", score),
 			   log_data_integer("score", e->score));
+#endif
+		e->score = score;
+		reputation_changed_update_users(e);
+	} else
+	if (e && forced)
+	{
+#ifdef DEBUGMODE
+		unreal_log(ULOG_DEBUG, "reputation", "REPUTATION_DECREASE", client,
+			   "Reputation score for for $ip from $client is $score, force-setting to $their_score.",
+			   log_data_string("ip", ip),
+			   log_data_integer("their_score", score),
+			   log_data_integer("score", e->score));
 #endif
 		e->score = score;
 		reputation_changed_update_users(e);
@@ -1320,11 +1412,12 @@ CMD_FUNC(reputation_server_cmd)
 
 	/* Propagate to the non-client direction (score may be updated) */
 	sendto_server(client, 0, 0, NULL,
-	              ":%s REPUTATION %s %s%d",
+	              ":%s REPUTATION %s %s%d%s",
 	              client->id,
 	              parv[1],
 	              allow_reply ? "" : "*",
-	              score);
+	              score,
+	              forced ? "*" : "");
 }
 
 CMD_FUNC(reputation_cmd)
@@ -1377,3 +1470,72 @@ int reputation_starttime_callback(void)
 	/* NB: fix this by 2038 */
 	return (int)reputation_starttime;
 }
+
+void _ban_act_set_reputation(Client *client, BanAction *action)
+{
+	ReputationEntry *e;
+	int value;
+
+	if ((client->ip == NULL) || IsDead(client))
+		return; /* Wait, this is impossible, right? */
+
+	/* There's a problem with adjusting reputation scores
+	 * when a user is in "unknown state", especially when decreasing
+	 * it, which is the common case here.
+	 * This is because we might be a new server with not much
+	 * reputation scores in the database, so giving everyone a 0,
+	 * this is automatically fixed post-connect by other servers
+	 * syncing in normally.
+	 * However, if we allow setting/decreasing reputation here in
+	 * this function, before the adjustment happens, then we may be
+	 * decreasing/setting to a very low value (because we think
+	 * the user has 0 rep) while in fact the user has high rep
+	 * and should still have high rep (just a little lower than before).
+	 * Instead of breaking my head on this, let's just not deal with
+	 * unknown users for now. Can always enhance this later.
+	 */
+	if (!IsUser(client))
+		return;
+
+	e = find_reputation_entry(client->ip);
+	if (!e)
+	{
+		/* Create */
+		e = safe_alloc(sizeof(ReputationEntry)+strlen(client->ip));
+		strcpy(e->ip, client->ip); /* safe, allocated above */
+		add_reputation_entry(e);
+	}
+
+	value = e->score;
+
+	switch (action->var_action)
+	{
+		case VAR_ACT_SET:
+			value = action->value;
+			break;
+		case VAR_ACT_INCREASE:
+			value += action->value;
+			if (value > REPUTATION_SCORE_CAP)
+				value = REPUTATION_SCORE_CAP;
+			break;
+		case VAR_ACT_DECREASE:
+			value -= action->value;
+			if (value < 0)
+				value = 0;
+			break;
+		default:
+			abort();
+	}
+
+	/* Nothing changed? Then we are done. */
+	if (e->score == value)
+		return;
+
+	e->score = value;
+	reputation_changed_update_users(e);
+	sendto_server(&me, 0, 0, NULL,
+	              ":%s REPUTATION %s *%d*",
+	              me.id,
+	              e->ip,
+	              e->score);
+}
