@@ -100,9 +100,10 @@ RPC_CALL_FUNC(rpc_spamfilter_list)
 
 /* Shared code for selecting a spamfilter, for .add/.del/get */
 int spamfilter_select_criteria(Client *client, json_t *request, json_t *params, const char **name, int *match_type,
-                               int *targets, char *targetbuf, size_t targetbuflen, BanAction *action, char *actionbuf)
+                               int *targets, char *targetbuf, size_t targetbuflen, BanActionValue *action, char *actionbuf)
 {
 	const char *str;
+	int actval;
 
 	*name = json_object_get_string(params, "name");
 	if (!*name)
@@ -150,6 +151,12 @@ int spamfilter_select_criteria(Client *client, json_t *request, json_t *params,
 		rpc_error(client, request, JSON_RPC_ERROR_INVALID_PARAMS, "Invalid value for parameter 'ban_action'");
 		return 0;
 	}
+	/* I hope to remove this restriction at a later point: */
+	if (banact_config_only(*action))
+	{
+		rpc_error(client, request, JSON_RPC_ERROR_INVALID_PARAMS, "Invalid value for parameter 'ban_action': action is config-based only");
+		return 0;
+	}
 	actionbuf[0] = banact_valtochar(*action);
 	actionbuf[1] = '\0';
 	return 1;
@@ -158,10 +165,9 @@ int spamfilter_select_criteria(Client *client, json_t *request, json_t *params,
 RPC_CALL_FUNC(rpc_spamfilter_get)
 {
 	json_t *result;
-	int type = TKL_SPAMF|TKL_GLOBAL;
 	const char *name;
 	TKL *tkl;
-	BanAction action;
+	BanActionValue action;
 	int match_type = 0;
 	int targets = 0;
 	char targetbuf[64];
@@ -170,11 +176,16 @@ RPC_CALL_FUNC(rpc_spamfilter_get)
 	if (!spamfilter_select_criteria(client, request, params, &name, &match_type, &targets, targetbuf, sizeof(targetbuf), &action, actionbuf))
 		return; /* Error already communicated to client */
 
-	tkl = find_tkl_spamfilter(type, name, action, targets);
+	/* For spamfilter.get we first try global spamfilters, then local. */
+	tkl = find_tkl_spamfilter(TKL_SPAMF|TKL_GLOBAL, name, action, targets);
 	if (!tkl)
 	{
-		rpc_error(client, request, JSON_RPC_ERROR_NOT_FOUND, "Spamfilter not found");
-		return;
+		tkl = find_tkl_spamfilter(TKL_SPAMF, name, action, targets);
+		if (!tkl)
+		{
+			rpc_error(client, request, JSON_RPC_ERROR_NOT_FOUND, "Spamfilter not found");
+			return;
+		}
 	}
 
 	result = json_object();
@@ -193,7 +204,7 @@ RPC_CALL_FUNC(rpc_spamfilter_add)
 	time_t ban_duration = 0;
 	TKL *tkl;
 	Match *m;
-	BanAction action;
+	BanActionValue action;
 	int match_type = 0;
 	int targets = 0;
 	char targetbuf[64];
@@ -225,7 +236,7 @@ RPC_CALL_FUNC(rpc_spamfilter_add)
 
 	OPTIONAL_PARAM_STRING("set_by", set_by);
 	if (!set_by)
-		set_by = client->name;
+		set_by = client->rpc->issuer ? client->rpc->issuer : client->name;
 
 	if (find_tkl_spamfilter(type, name, action, targets))
 	{
@@ -246,8 +257,8 @@ RPC_CALL_FUNC(rpc_spamfilter_add)
 		return;
 	}
 
-	tkl = tkl_add_spamfilter(type, targets, action, m, set_by, 0, TStime(),
-	                         ban_duration, reason, 0);
+	tkl = tkl_add_spamfilter(type, NULL, targets, banact_value_to_struct(action), m, NULL, NULL, set_by, 0, TStime(),
+	                         ban_duration, reason, INPUT_CONVERSION_DEFAULT, 0, 0);
 
 	if (!tkl)
 	{
@@ -270,7 +281,7 @@ RPC_CALL_FUNC(rpc_spamfilter_del)
 	const char *name;
 	const char *set_by;
 	TKL *tkl;
-	BanAction action;
+	BanActionValue action;
 	int match_type = 0;
 	int targets = 0;
 	char targetbuf[64];
@@ -282,7 +293,7 @@ RPC_CALL_FUNC(rpc_spamfilter_del)
 
 	OPTIONAL_PARAM_STRING("set_by", set_by);
 	if (!set_by)
-		set_by = client->name;
+		set_by = client->rpc->issuer ? client->rpc->issuer : client->name;
 
 	tkl = find_tkl_spamfilter(type, name, action, targets);
 	if (!tkl)
@@ -308,7 +319,7 @@ RPC_CALL_FUNC(rpc_spamfilter_del)
 	tkllayer[11] = name;
 	tkllayer[12] = NULL;
 
-	cmd_tkl(&me, NULL, 12, tkllayer);
+	cmd_tkl(NULL, &me, NULL, 12, tkllayer);
 
 	tkl = find_tkl_spamfilter(type, name, action, targets);
 	if (!tkl)
